<?php

namespace Drupal\mirotalk_sfu\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Extension\ExtensionPathResolver;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\StreamWrapper\PublicStream;
use Drupal\Core\Url;
use Drupal\file\Entity\File;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Miro talk sfu controller.
 */
class MirotalkSfuController extends ControllerBase {

  /**
   * The controller miro talk sfu.
   *
   * @param \Drupal\Core\File\FileSystemInterface $fileSystem
   *   The file system service.
   * @param \Drupal\Core\Extension\ExtensionPathResolver $pathResolver
   *   Extension path service.
   * @param \Drupal\Core\Routing\RouteMatchInterface $routeMatch
   *   Route match service.
   */
  public function __construct(
    protected FileSystemInterface $fileSystem,
    protected ExtensionPathResolver $pathResolver,
    protected RouteMatchInterface $routeMatch,
  ) {
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('file_system'),
      $container->get('extension.path.resolver'),
      $container->get('current_route_match'),
    );
  }

  /**
   * {@inheritDoc}
   */
  public function __invoke(): array {
    $roomId = $this->routeMatch->getParameter('roomId');

    $entity_storage = $this->entityTypeManager()->getStorage('meeting');
    $query = $entity_storage->getQuery()
      ->condition('label', $roomId)
      ->accessCheck(FALSE);
    $entity_ids = $query->execute();
    if (empty($entity_ids)) {
      throw new NotFoundHttpException();
    }

    $config = $this->config('mirotalk_sfu.settings')->getRawData();
    $module_path = $this->pathResolver->getPath('module', 'mirotalk_sfu');
    return [
      "#theme" => "meeting_room",
      "#attached" => [
        "library" => [
          "mirotalk_sfu/meeting_room",
        ],
        "drupalSettings" => [
          'mirotalk_sfu' => $config,
          'modulePath' => base_path() . $module_path,
        ],
      ],
      "#data" => $module_path,
      "#base_path" => base_path(),
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function recording(Request $request) {
    $config = $this->config('mirotalk_sfu.settings');

    $fileName = $request->query->get('fileName');
    if (!$fileName) {
      return new JsonResponse(['error' => 'Missing fileName'], 400);
    }

    $path_video = [DRUPAL_ROOT, PublicStream::basePath(), $config->get('video_storage_path')];
    $path_video = implode('/', $path_video);

    $this->fileSystem->prepareDirectory($path_video, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);
    $filePath = $path_video . '/' . $fileName;

    $content = $request->getContent();
    if (!empty($content)) {
      file_put_contents($filePath, $content, FILE_APPEND);
      return new JsonResponse(['status' => 'success', 'file' => $filePath]);
    }

    return new JsonResponse(['error' => 'No data received'], 400);
  }

  /**
   * {@inheritDoc}
   */
  public function createMeeting(Request $request) {
    $room_id = $request->request->get('room_id');
    $entity_type = $request->request->get('entity_type');
    $entity_id = $request->request->get('entity_id');
    $field_name = $request->request->get('field_name');
    $room_type = $request->request->get('room_type');

    try {
      $entity_storage = $this->entityTypeManager()->getStorage('meeting');
      $query = $entity_storage->getQuery()
        ->condition('label', $room_id)
        ->accessCheck(FALSE);
      $entity_ids = $query->execute();

      if (empty($entity_ids)) {
        $mirotalk = $this->entityTypeManager()->getStorage('meeting')->create(values: ['label' => $room_id]);
        $mirotalk->save();

        if ($room_type === 'node') {
          /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
          $entity = $this->entityTypeManager()->getStorage($entity_type)->load($entity_id);

          if (!$entity) {
            return new JsonResponse([
              'success' => FALSE,
              'message' => $this->t('Entity of type @entity_type with ID @entity_id not found.', [
                '@entity_type' => $entity_type,
                '@entity_id' => $entity_id,
              ]),
            ]);
          }

          if (!$entity->hasField($field_name)) {
            return new JsonResponse([
              'success' => FALSE,
              'message' => $this->t('Field @field_name does not exist on the entity.', [
                '@field_name' => $field_name,
              ]),
            ]);
          }

          $field_meeting = $entity->get($field_name)->getValue();
          $field_meeting[] = ['target_id' => $mirotalk->id()];
          $entity->set($field_name, $field_meeting);
          $entity->save();
        }
      }
      else {
        $entities = $entity_storage->loadMultiple($entity_ids);
        foreach ($entities as $entity) {
          /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
          $entity->set('status', 1);
          $entity->save();
        }
      }

      return new JsonResponse([
        'success' => TRUE,
        'meeting_url' => Url::fromRoute('mirotalk_sfu.meeting', ['roomId' => $room_id])->toString(),
      ]);
    }
    catch (\Exception $e) {
      return new JsonResponse([
        'success' => FALSE,
        'message' => $e->getMessage(),
      ]);
    }
  }

  /**
   * {@inheritDoc}
   */
  public function apiOffVideo(Request $request) {
    $data = json_decode($request->getContent(), TRUE);
    if ($data == NULL) {
      return new JsonResponse([
        'success' => FALSE,
      ]);
    }
    $entity_storage = $this->entityTypeManager()->getStorage('meeting');
    $query = $entity_storage->getQuery()
      ->condition('label', $data['room_id'])
      ->accessCheck(FALSE);
    $entity_ids = $query->execute();
    if (!empty($entity_ids)) {
      $entities = $entity_storage->loadMultiple($entity_ids);
      foreach ($entities as $entity) {
        /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
        $entity->set('status', 0);
        $entity->save();
      }
      return new JsonResponse([
        'success' => TRUE,
      ]);
    }
    return new JsonResponse([
      'success' => FALSE,
    ]);
  }

  /**
   * {@inheritDoc}
   */
  public function apiUserJoin(Request $request) {
    $data = json_decode($request->getContent(), TRUE);
    if ($data == NULL) {
      return new JsonResponse([
        'success' => FALSE,
      ]);
    }

    $entity_storage = $this->entityTypeManager()->getStorage('meeting');
    $query = $entity_storage->getQuery()
      ->condition('label', $data['room_id'])
      ->accessCheck(FALSE);

    $entity_ids = $query->execute();
    if (!empty($entity_ids)) {
      $entities = $entity_storage->loadMultiple($entity_ids);

      foreach ($entities as $entity) {
        /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
        $participants = $entity->get('participants')->getValue();

        if (count($participants) > 0 && $participants[0]['value'] !== 'null') {
          $participants = json_decode($participants[0]['value'], TRUE);
          unset($participants[0]);
        }

        if (!array_key_exists($data['peer_uuid'], $participants)) {
          $participants[$data['peer_uuid']] = [
            'peer_name' => $data['peer_name'],
            'join_time' => $data['join_time'],
          ];

          $entity->set('participants', json_encode($participants));
          $entity->save();
        }
      }

      return new JsonResponse([
        'success' => TRUE,
      ], 200);
    }

    return new JsonResponse([
      'success' => FALSE,
    ], 500);
  }

  /**
   * {@inheritDoc}
   */
  public function apiChatRoom(Request $request) {
    $data = json_decode($request->getContent(), TRUE);
    if ($data == NULL) {
      return new JsonResponse([
        'success' => FALSE,
      ]);
    }

    $entity_storage = $this->entityTypeManager()->getStorage('meeting');
    $query = $entity_storage->getQuery()
      ->condition('label', $data['room_id'])
      ->accessCheck(FALSE);

    $entity_ids = $query->execute();
    if (!empty($entity_ids)) {
      $entities = $entity_storage->loadMultiple($entity_ids);

      foreach ($entities as $entity) {
        /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
        $chats = $entity->get('chat')->getValue();

        if (count($chats) > 0) {
          $chats = json_decode($chats[0]['value'], TRUE);
          $lastChat = end($chats);
          $lastKey = key($chats);
        }

        if ($chats && $data['peer_uuid'] === $lastChat['peer_uuid']) {
          $chats[$lastKey]['peer_content'][] = [
            'peer_msg' => $data['peer_msg'],
            'peer_time' => $data['peer_time'],
          ];
        }
        else {
          $chats[] = [
            'peer_uuid' => $data['peer_uuid'],
            'peer_name' => $data['peer_name'],
            'peer_content' => [
              [
                'peer_msg' => $data['peer_msg'],
                'peer_time' => $data['peer_time'],
              ],
            ],
          ];
        }

        $entity->set('chat', json_encode($chats));
        $entity->save();
      }

      return new JsonResponse([
        'success' => TRUE,
      ], 200);
    }

    return new JsonResponse([
      'success' => FALSE,
    ], 500);
  }

  /**
   * {@inheritDoc}
   */
  public function apiTranscription(Request $request) {
    $data = json_decode($request->getContent(), TRUE);
    if ($data == NULL) {
      return new JsonResponse([
        'success' => FALSE,
      ]);
    }

    $entity_storage = $this->entityTypeManager()->getStorage('meeting');
    $query = $entity_storage->getQuery()
      ->condition('label', $data['room_id'])
      ->accessCheck(FALSE);

    $entity_ids = $query->execute();
    if (!empty($entity_ids)) {
      $entities = $entity_storage->loadMultiple($entity_ids);

      foreach ($entities as $entity) {
        /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
        $content = '<p><strong>' . $data['peer_name'] . ':</strong>' . $data['text_data'] . '</p>';
        $current_content = $entity->get('description')->getValue();

        if ($current_content[0]['value'] !== 'null') {
          $content = $current_content[0]['value'] . $content;
        }
      }

      $entity->set('description', [
        'value' => $content,
        'format' => 'basic_html',
      ]);
      $entity->save();

      return new JsonResponse([
        'success' => TRUE,
      ], 200);
    }

    return new JsonResponse([
      'success' => FALSE,
    ], 500);
  }

  /**
   * {@inheritDoc}
   */
  public function apiAddVideo(Request $request) {
    $config = $this->config('mirotalk_sfu.settings');
    $data = json_decode($request->getContent(), TRUE);

    if ($data == NULL) {
      return;
    }

    $entity_storage = $this->entityTypeManager()->getStorage('meeting');
    $query = $entity_storage->getQuery()
      ->condition('label', $data['room_id'])
      ->accessCheck(FALSE);

    $entity_ids = $query->execute();

    $video_path = 'public://' . $config->get('video_storage_path') . '/' . $data['name'];
    $file_name = $this->fileSystem->realpath($video_path);

    if (file_exists($file_name) && !empty($entity_ids)) {
      $file = File::create([
        'uri' => $video_path,
        'status' => 1,
      ]);
      $file->save();

      $entities = $entity_storage->loadMultiple($entity_ids);

      foreach ($entities as $entity) {
        /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
        $entity->set('field_video', $file->id());
        $entity->save();
      }

      return new JsonResponse([
        'success' => TRUE,
      ]);
    }

    return new JsonResponse([
      'success' => FALSE,
    ]);
  }

}
