<?php

namespace Drupal\mirotalk_sfu\Form;

use Drupal\Core\Extension\ExtensionPathResolver;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configuration form for a meeting entity type.
 */
class MeetingSettingsForm extends ConfigFormBase {

  /**
   * The constructor of setting form.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   Module service.
   * @param \Drupal\Core\Extension\ExtensionPathResolver $pathResolver
   *   Extension path service.
   */
  public function __construct(protected ModuleHandlerInterface $moduleHandler, protected ExtensionPathResolver $pathResolver) {
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('module_handler'),
      $container->get('extension.path.resolver')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'meeting_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildForm($form, $form_state);
    $config = $this->config('mirotalk_sfu.settings');
    if (!empty($config->get('server_path'))) {
      $module_path = $config->get('server_path') . DIRECTORY_SEPARATOR . '.env';
    }
    if (!empty($module_path) && !file_exists($module_path)) {
      $this->messenger()->addError("Failed to load environment $module_path. Please check Readme.md");
      $module_path = $config->get('server_path') . DIRECTORY_SEPARATOR . '.env.template';
    }
    if (!empty($module_path) && !file_exists($module_path)) {
      $module_path = $this->pathResolver->getPath('module', 'mirotalk_sfu') . DIRECTORY_SEPARATOR . '.env';
    }
    $fileDefault = file_get_contents($module_path) ?? '';

    $form['server_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Server Path'),
      '#default_value' => $config->get('server_path') ?? '',
      '#description' => $this->t("The path to the server's MiroTalk SFU source. Ex: @local", ['@local' => '/opt/mirotalk/']),
      '#required' => TRUE,
    ];

    $form['server_url'] = [
      '#type' => 'textfield',
      '#default_value' => $config->get('server_url') ?? '',
      '#required' => TRUE,
      '#access' => FALSE,
    ];

    $form['video_storage_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Video Storage Path'),
      '#default_value' => $config->get('video_storage_path') ?? '',
      '#description' => $this->t('Folder name to store video (/default/files/%folder)', ['%folder' => $config->get('video_storage_path')]),
      '#required' => TRUE,
    ];

    $form['config_server'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Config server'),
      '#default_value' => $fileDefault,
      '#description' => $this->t('Load the mirotalksfu environment file %directory/.env', ['%directory' => $config->get('server_path')]),
      '#attributes' => [
        'data-json-editor' => 'form',
        'data-id' => 'config-server',
        'class' => ['yaml-editor'],
      ],
      '#attached' => [
        'library' => ['mirotalk_sfu/yaml_editor'],
      ],
      '#prefix' => '<div id="' . $this->getFormId() . '-list">',
      '#suffix' => '</div>',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['mirotalk_sfu.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $config = $this->configFactory()->getEditable('mirotalk_sfu.settings');

    $server_config = $form_state->getValue('config_server');
    $server_path = $form_state->getValue('server_path') . DIRECTORY_SEPARATOR . '.env';
    $server_url = '';
    if (preg_match('/^SERVER_HOST_URL\s*=\s*(.*?)\s*(#.*)?$/m', $server_config, $matches)) {
      $server_url = trim($matches[1] ?? '');
    }

    $config->set('server_path', $form_state->getValue('server_path'))
      ->set('server_url', $server_url)
      ->set('video_storage_path', $form_state->getValue('video_storage_path'))
      ->save();

    parent::submitForm($form, form_state: $form_state);
    $this->setupServer($server_path, $server_config);
  }

  /**
   * Setup server.
   */
  protected function setupServer($server_path, $configuration): void {
    if (empty($server_path)) {
      $this->messenger()->addError('Server path is missing.');
      return;
    }

    if (file_put_contents($server_path, $configuration)) {
      // Restart server.
      $output = [];
      exec('pm2 restart "Miro Talk"', $output, $return_var);
      if ($return_var !== 0) {
        $this->logger('mirotalk_sfu')->error('Failed to restart the server. Output: ' . implode("\n", $output));
      }
      else {
        $this->messenger()->addStatus($this->t('Server restarted successfully.'));
      }
    }
    else {
      $this->messenger()->addError("Failed to write to {$server_path}.");
    }
  }

}
