<?php

namespace Drupal\mirotalk_sfu\Plugin\Block;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\Attribute\Block;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a meeting block.
 */
#[Block(
  id: "meeting_block",
  admin_label: new TranslatableMarkup("Meeting"),
  category: new TranslatableMarkup('Conference'),
)]
class MeetingBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Creates a MeetingsBlock instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager interface.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, protected EntityTypeManagerInterface $entityTypeManager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'room_id' => '',
      'video_path' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state): array {
    $form['room_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Room Id'),
      '#default_value' => $this->configuration['room_id'],
    ];
    $form['video_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Video Path'),
      '#default_value' => $this->configuration['video_path'],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state): void {
    $this->configuration['room_id'] = $form_state->getValue('room_id');
    $this->configuration['video_path'] = $form_state->getValue('video_path');
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    $settings = $this->configuration;

    $query = $this->entityTypeManager->getStorage('meeting')->getQuery()
      ->condition('created', strtotime('now') - 3600, '>')
      ->condition('status', 1)
      ->sort('created', 'DESC')
      ->accessCheck(FALSE);

    $meeting_ids = $query->execute();

    $meetings = $this->entityTypeManager->getStorage('meeting')->loadMultiple($meeting_ids);

    $rooms = [];
    foreach ($meetings as $meeting) {
      /** @var \Drupal\Core\Entity\ContentEntityInterface $meeting */
      $participants = $meeting->get('participants')->getValue();
      $countParticipant = 0;

      if (count($participants) > 0 && $participants[0]['value'] !== 'null') {
        $countParticipant = count(json_decode($participants[0]['value'], TRUE));
      }

      $rooms[] = [
        'id' => $meeting->label(),
        'count' => $countParticipant,
      ];
    }

    if (empty($settings['room_id'])) {
      $characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
      $part1 = substr(str_shuffle($characters), 0, 3);
      $part2 = substr(str_shuffle($characters), 0, 4);
      $part3 = substr(str_shuffle($characters), 0, 3);
      $settings['room_id'] = implode('-', [$part1, $part2, $part3]);
    }
    $settings['url'] = Url::fromRoute('mirotalk_sfu.create_meeting')->toString();
    $settings['room_type'] = 'block';
    $build['content'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['mirotalk']],
      '#children' => [
        'create' => [
          '#type' => 'html_tag',
          '#tag' => 'button',
          '#value' => implode(' ', ['<i class="bi bi-camera-video"></i>', $this->t('Meeting')]),
          '#attributes' => [
            'class' => ['btn'],
            'data-bs-toggle' => 'modal',
            'data-bs-target' => '#listRoom',
          ],
        ],
      ],
      '#attached' => [
        'library' => ['mirotalk_sfu/join_room'],
        'drupalSettings' => ['create_room' => $settings],
      ],
    ];

    $build['content']['#children']['rooms'] = [
      '#type' => 'html_tag',
      '#tag' => 'div',
      '#theme' => 'modal_room',
      '#attributes' => [
        'id' => 'listRoom',
        'class' => ['modal', 'fade', 'list-room'],
        'tabindex' => '-1',
        'aria-labelledby' => "listRoomLabel",
        'aria-hidden' => 'true',
      ],
      '#title' => $this->t('Chat Rooms'),
      '#items' => $rooms,
      '#room_id' => $settings['room_id'],
      "#base_path" => base_path(),
    ];

    $build['#cache']['max-age'] = 0;
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account): AccessResult {
    // @todo Evaluate the access condition here.
    return AccessResult::allowedIf(TRUE);
  }

}
