<?php

namespace Drupal\mirotalk_sfu\Plugin\Field\FieldFormatter;

use Drupal\Component\Render\PlainTextOutput;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Field\Attribute\FieldFormatter;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Url;
use Drupal\Core\Utility\Token;
use Drupal\field\Entity\FieldConfig;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'button_join_room'.
 */
#[FieldFormatter(
  id: 'button_join_room',
  label: new TranslatableMarkup('Join Room for meeting'),
  description: new TranslatableMarkup('Button join room'),
  field_types: [
    'entity_reference',
  ],
)]
class MirotalkSfuFormatterJoin extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, protected EntityFieldManagerInterface $entityFieldManager, protected Token $token) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('entity_field.manager'),
      $container->get('token')
    );
  }

  /**
   * Apply only meeting entity.
   *
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    return ($field_definition->getFieldStorageDefinition()->getSetting('target_type') == 'meeting');
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings(): array {
    $setting = [
      'thumbnail' => '',
      'video' => '',
    ];
    return $setting + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {

    $entity_type = 'meeting';
    $bundle = 'meeting';
    $fieldDefinitions = $this->entityFieldManager->getFieldDefinitions($entity_type, $bundle);
    $options = [];
    // Supported field types.
    $typeSupport = ['image', 'file'];
    foreach ($fieldDefinitions as $fieldDefinition) {
      if ($fieldDefinition instanceof FieldConfig) {
        $type = $fieldDefinition->getType();
        if (in_array($type, $typeSupport)) {
          $options[$type][$fieldDefinition->getName()] = $fieldDefinition->label();
        }
      }
    }

    // Add location field select element if options are available.
    if (!empty($options['image'])) {
      $elements['thumbnail'] = [
        '#type' => 'select',
        '#title' => $this->t('Thumbnail video field'),
        '#default_value' => $this->getSetting('thumbnail'),
        '#options' => $options['image'],
        "#empty_option" => $this->t('- Select field -'),
        "#description" => $this->t('You must attach a image field to store the video thumbnail'),
      ];
    }
    if (!empty($options['file'])) {
      $elements['video'] = [
        '#type' => 'select',
        '#title' => $this->t('Video field'),
        '#default_value' => $this->getSetting('video'),
        '#options' => $options['file'],
        "#empty_option" => $this->t('- Select field -'),
        "#description" => $this->t('You must attach a file field to record the video'),
      ];
    }
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(): array {
    $fields = [];
    if (!empty($this->getSetting('thumbnail'))) {
      $fields[] = $this->t('Thumbnail field: @thumbnail', ['@thumbnail' => $this->getSetting('thumbnail')]);
    }
    if (!empty($this->getSetting('video'))) {
      $fields[] = $this->t('Video field: @video', ['@video' => $this->getSetting('video')]);
    }
    if (empty($fields)) {
      return [$this->t('You need to add a thumbnail and video field')];
    }
    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $title = $this->t('Create room');
    $class = 'btn-primary';
    $entity_type = 'meeting';
    $bundle = 'meeting';
    $fieldDefinitions = $this->entityFieldManager->getFieldDefinitions($entity_type, $bundle);
    $last_room = $items->get($items->count() - 1)?->entity;
    $room_id = implode('-', [
      $items->getEntity()->id(),
      date('d-m'),
    ]);
    $list_meeting = [];
    $options = ['clear' => TRUE];
    $token_data = [];
    $thumbnail_field = $this->getSetting('thumbnail');
    $video_field = $this->getSetting('video');
    $videoSetting = [
      'entity_type' => $items->getEntity()->getEntityTypeId(),
      'entity_id' => $items->getEntity()->id(),
      'field_name' => $items->getName(),
      'room_id' => $room_id,
      'room_type' => 'node',
      'room_status' => 0,
      'url' => Url::fromRoute('mirotalk_sfu.create_meeting')->toString(),
    ];
    if (!empty($thumbnail_field)) {
      $thumbnail_field = $fieldDefinitions[$thumbnail_field];
      $file_directory = trim($thumbnail_field->getSetting('file_directory'), '/');
      $videoSetting['thumbnail'] = PlainTextOutput::renderFromHtml($this->token->replace($file_directory, $token_data, $options));
    }
    if (!empty($video_field)) {
      $video_field = $fieldDefinitions[$video_field];
      $file_directory = trim($video_field->getSetting('file_directory'), '/');
      $videoSetting['video'] = PlainTextOutput::renderFromHtml($this->token->replace($file_directory, $token_data, $options));
    }
    if ($last_room && $last_room->get('status')->value === '1') {
      $title = $this->t('Join room');
      $class = 'btn-warning';
      $videoSetting['room_status'] = 1;
      $videoSetting['room_id'] = $last_room->label();
    }
    for ($i = 0; $i < $items->count(); $i++) {
      $entity = $items->get($i)?->entity;
      $created = $entity->get('created')?->value ?? strtotime('now');
      $data = [
        'entity_url' => $entity->toUrl()->toString(),
        'entity_label' => $entity->label(),
        'entity_created' => $created,
      ];
      array_push($list_meeting, $data);
    }
    return [
      '#type' => 'container',
      '#attributes' => ['class' => ['mirotalk-project']],
      '#children' => [
        'join' => [
          '#type' => 'html_tag',
          '#tag' => 'button',
          '#value' => $title,
          '#attributes' => [
            'class' => ['create-room-project', 'btn', $class],
            'data-room_id' => $room_id,
          ],
        ],
        'list' => [
          '#theme' => 'mirotalk_sfu_meeting',
          '#content' => $this->t('History'),
          '#data' => array_reverse($list_meeting),
          "#base_path" => base_path(),
        ],
      ],
      '#attached' => [
        'library' => ['mirotalk_sfu/join_room'],
        'drupalSettings' => ['mirotalk_sfu' => $videoSetting],
      ],
    ];
  }

}
